@@ -23,18 +23,24 @@
 #' @param prop_missing Proportion of responses that should be missing. Default
 #'   value is \code{0}. This argument is valid for only
 #'   \code{\link{Itempool-class}} and \code{\link{Testlet-class}} objects.
+#' @param output Type of the output. Following options are available:
+#'     \describe{
+#'       \item{\code{"matrix"}}{A \code{matrix} object.}
+#'       \item{\code{"response_set"}}{A \code{\link{Response_set-class}}
+#'             object with item pool attached.}
+#'     }
 #'
-#' @include Item-class.R
-#' @include Item-class-methods.R
-#' @include Itempool-class.R
-#' @include Itempool-class-methods.R
+#' @include item-class.R
+#' @include item-class-methods.R
+#' @include itempool-class.R
+#' @include itempool-class-methods.R
 #'
 #' @return A vector of responses.
 #'
 #' @author Emre Gonulates
 #'
-setGeneric("sim_resp", function(ip, theta, prop_missing = 0)
-{standardGeneric ("sim_resp")})
+setGeneric("sim_resp", function(ip, theta, prop_missing = 0, output = "matrix")
+{standardGeneric("sim_resp")})
 
 
 ###############################################################################@
@@ -58,14 +64,13 @@ setGeneric("sim_resp", function(ip, theta, prop_missing = 0)
 #'
 setMethod(
   f = "sim_resp", signature = c(ip = "Item"),
-  function(ip, theta, prop_missing = 0){
+  function(ip, theta, prop_missing = 0, output = "matrix") {
     # If the Model is one of the following: "irt1PM" "irt2PM" "irt3PM" "irt4PM"
-    if (ip$model %in%
-          names(Pmodels)[sapply(Pmodels, function(x) x$model_family == 'UIRT')])
+    if (ip$model %in% UNIDIM_DICHO_MODELS)
     {
       u <- runif(length(theta))
-      P <- prob(ip = ip, theta = theta)
-      return(stats::setNames(as.integer(u<P), names(theta)))
+      P <- prob(ip = ip, theta = theta)[, 2]
+      return(stats::setNames(as.integer(u < P), names(theta)))
     } else if (ip$model %in% c("PCM", "GRM", "GPCM", "GPCM2")) {
       # This method is based on De Ayala (1994) The Influence of
       # Multidimensionality on the, p. 158-159: "The generation of an examinee’s
@@ -104,9 +109,9 @@ setMethod(
 #' sim_resp(ip = testlet, theta = rnorm(1))
 setMethod(
   f = "sim_resp", signature = c(ip = "Testlet"),
-  function(ip, theta, prop_missing = 0){
+  function(ip, theta, prop_missing = 0, output = "matrix"){
     ip <- ip@item_list
-    return(sim_resp(ip, theta, prop_missing = prop_missing))
+    return(sim_resp(ip, theta, prop_missing = prop_missing, output = output))
   }
 )
 
@@ -137,34 +142,41 @@ setMethod(
 #' sim_resp(ip = ip, theta = rnorm(5), prop_missing = .1)
 setMethod(
   f = "sim_resp", signature = c(ip = "Itempool"),
-  function(ip, theta, prop_missing = 0){
-    # If the Model is one of the following: "irt1PM" "irt2PM" "irt3PM" "irt4PM"
-    if (all(ip$model %in% c(names(Pmodels)[sapply(Pmodels, function(x)
-      x$model_family == 'UIRT')], "GRM", "GPCM", "GPCM2", "PCM", "BTM")))
-    {
-      result <- sapply(ip@item_list, FUN =
-                         function(x) sim_resp(ip = x, theta = theta))
-      col_names <- ip$id
-      if (!is.matrix(result)) {
-        if (is.list(result)) { # If ip has testlets,then result should be a list
-          col_names <- ip$resp_id
-          result <- unlist(result)
-          # return(matrix(unlist(result), nrow = length(theta),
-          #               dimnames = list(names(theta), ip$resp_id)))
+  function(ip, theta, prop_missing = 0, output = "matrix") {
+    if (output == "matrix") {
+      # If the Model is one of the following: "Rasch" "1PM" "2PM" "3PM" "4PM"
+      if (all(ip$model %in% c(UNIDIM_DICHO_MODELS, UNIDIM_POLY_MODELS, "BTM")))
+      {
+        result <- sapply(ip@item_list, FUN =
+                           function(x) sim_resp(ip = x, theta = theta))
+        col_names <- ip$item_id
+        if (!is.matrix(result)) {
+          # If ip has testlets, then result should be a list
+          if (is.list(result)) {
+            col_names <- ip$resp_id
+            result <- unlist(result)
+            # return(matrix(unlist(result), nrow = length(theta),
+            #               dimnames = list(names(theta), ip$resp_id)))
+          }
+          result <- matrix(result, nrow = length(theta))
+          # result <- matrix(result, nrow = length(theta),
+          #                  ncol = length(ip@item_list))
         }
-        result <- matrix(result, nrow = length(theta))
-        # result <- matrix(result, nrow = length(theta),
-        #                  ncol = length(ip@item_list))
-      }
 
-      result[sample(1:length(result), round(length(result)*prop_missing))] <- NA
-      # Set column and row names or resp matrix.
-      if (is.null(names(theta))) {
-        rownames(result) <- paste0("S", 1:length(theta))
-        } else rownames(result) <- names(theta)
-      colnames(result) <- col_names
-      return(result)
-    } else stop("This model is not implemented in 'sim_resp' function.")
+        result[sample(1:length(result), round(length(result)*prop_missing))] <- NA
+        # Set column and row names or resp matrix.
+        if (is.null(names(theta))) {
+          rownames(result) <- paste0("S", 1:length(theta))
+          } else rownames(result) <- names(theta)
+        colnames(result) <- col_names
+        return(result)
+      } else stop("This model is not implemented in 'sim_resp' function.")
+    } else if (output == "response_set") {
+      return(sim_resp_response_set_cpp(theta = theta, ip = ip,
+                                       prop_missing = prop_missing))
+    } else
+      stop(paste0("Invalid 'output'. 'output' argument can only be either ",
+                  "\"matrix\" or \"response_set\"."), call. = FALSE)
   }
 )
 
@@ -178,10 +190,11 @@ setMethod(
 #'
 setMethod(
   f = "sim_resp", signature = c(ip = "numMatDfListChar"),
-  function(ip, theta){
+  function(ip, theta, prop_missing = 0, output = "matrix"){
     if (inherits(ip, "numeric")) {
       tryCatch({
-        return(sim_resp(ip = itempool(ip), theta = theta))
+        return(sim_resp(ip = itempool(ip), theta = theta,
+                        prop_missing = prop_missing, output = output))
       }, error = function(e) {
         stop("Cannot convert object to an 'Item' object. Please provide a ",
              "valid object using 'item()' function. \nThe reason for ",
@@ -189,7 +202,8 @@ setMethod(
       })
     } else if (inherits(ip, c("matrix", "data.frame", "list"))) {
       tryCatch({
-        return(sim_resp(ip = itempool(ip), theta = theta))
+        return(sim_resp(ip = itempool(ip), theta = theta,
+                        prop_missing = prop_missing, output = output))
       }, error = function(e) {
         stop("Cannot convert object to an 'Itempool' object. Please ",
              "provide a valid object using 'itempool()' function. \nThe ",
