@@ -50,7 +50,7 @@ Rcpp::NumericVector avg_rank(Rcpp::NumericVector x)
 
 
 //#############################################################################@
-//########################### cat_sim_single_cpp ###############################
+//########################### biserial_cpp #####################################
 //#############################################################################@
 // [[Rcpp::export]]
 double biserial_cpp(Rcpp::NumericVector score,
@@ -72,7 +72,7 @@ double biserial_cpp(Rcpp::NumericVector score,
     criterion = avg_rank(criterion);
     type = "brogden";
   }
-  if ((type == "clemans-lord") | (type == "brogden")) {
+  if ((type == "clemans-lord") || (type == "brogden")) {
     // Calculate deviation
     dev = criterion - mean(criterion);
     NumericVector sorted_score = clone(score);
